/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.jdbc.dao;

import org.wso2.registry.jdbc.DatabaseConstants;
import org.wso2.registry.LogEntry;

import java.sql.*;
import java.util.List;
import java.util.ArrayList;

public class RatingsDAO {

    public void addRating(long artifactID, String userID, int rating, Connection conn)
            throws SQLException {

        String sql = "INSERT INTO RATINGS (AID, USER_ID, RATING, RATED_TIME) " +
                "VALUES (?, ?, ?, ?)";

        long now = System.currentTimeMillis();

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, artifactID);
        s.setString(2, userID);
        s.setInt(3, rating);
        s.setDate(4, new Date(now));

        s.executeUpdate();
    }

    public void updateRating(long artifactID, String userID, int rating, Connection conn)
            throws SQLException {

        String sql = "UPDATE RATINGS SET RATING=?, RATED_TIME=? WHERE AID=? AND USER_ID=?";

        long now = System.currentTimeMillis();

        PreparedStatement s = conn.prepareStatement(sql);
        s.setInt(1, rating);
        s.setDate(2, new Date(now));
        s.setLong(3, artifactID);
        s.setString(4, userID);

        s.executeUpdate();
    }

    public boolean ratingExist(long artifactID, String userID, Connection conn)
            throws SQLException {

        String sql = "SELECT COUNT(R.R_ID) FROM RATINGS R WHERE R.AID=? AND R.USER_ID=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, artifactID);
        s.setString(2, userID);

        ResultSet result = s.executeQuery();
        long count = 0;
        if (result.next()) {
            count = result.getLong(1);
        }

        return count == 1? true: false;
    }

    public int getAverageRating(String path, Connection conn) throws SQLException {

        String sql = "SELECT AVG(R.RATING) FROM RATINGS R, ARTIFACTS A WHERE A.PATH=? AND A.AID=R.AID";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, path);

        ResultSet result = s.executeQuery();

        int averageRating = 0;
        if (result.next()) {
            averageRating = result.getInt(1);
        }

        return averageRating;
    }

    public int getRating(String path, String userName, Connection conn) throws SQLException {

        String sql = "SELECT R.RATING FROM RATINGS R, ARTIFACTS A WHERE A.PATH=? AND R.USER_ID=? AND A.AID=R.AID";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, path);
        s.setString(2, userName);

        ResultSet result = s.executeQuery();

        int rating = 0;
        if (result.next()) {
            rating = result.getInt(1);
        }

        return rating;
    }

    public void removeRatings(long artifactID, Connection conn) throws SQLException {

        String sql = "DELETE FROM RATINGS WHERE AID=?";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setLong(1, artifactID);
        s.executeUpdate();
    }

    public String[] getRatedUserNames(String resourcePath, Connection conn) throws SQLException {

        String sql = "SELECT R.USER_ID FROM RATINGS R, ARTIFACTS A WHERE A.PATH=? AND A.AID=R.AID";

        PreparedStatement s = conn.prepareStatement(sql);
        s.setString(1, resourcePath);

        List userNameList = new ArrayList();
        ResultSet results = s.executeQuery();
        while(results.next()) {
            String ratedUserName = results.getString(DatabaseConstants.USER_ID_FIELD);
            userNameList.add(ratedUserName);
        }

        String[] userNames = new String[userNameList.size()];
        for (int i = 0; i < userNames.length; i++) {
            userNames[i] = ((String) userNameList.get(i));
        }

        return userNames;
    }

    public List getLogs(String resourcePath, String userName,
                        java.util.Date from, java.util.Date to, Connection conn)
            throws SQLException {

        ArtifactDAO artifactDAO = new ArtifactDAO();

        long aid = -1;
        aid = artifactDAO.getArtifactID(resourcePath, conn);

        String sql = "SELECT A.PATH, R.USER_ID, R.RATED_TIME, R.RATING FROM ARTIFACTS A, RATINGS R WHERE A.AID=R.AID";

        if (aid != -1) {
            sql = sql + " AND R.AID=?";
        }

        if (userName != null) {
            sql = sql + "  AND R.USER_ID=?";
        }

        if (from != null) {
            sql = sql + " AND R.RATED_TIME>?";
        }

        if (to != null) {
            sql = sql + " AND R.RATED_TIME<?";
        }

        PreparedStatement s = conn.prepareStatement(sql);

        int paramNumber = 1;

        if (aid != -1) {
            s.setLong(paramNumber, aid);
            paramNumber++;
        }

        if (userName != null) {
            s.setString(paramNumber, userName);
            paramNumber++;
        }

        if (from != null) {
            s.setTimestamp(paramNumber, new Timestamp(from.getTime()));
            paramNumber++;
        }

        if (to != null) {
            s.setTimestamp(paramNumber, new Timestamp(to.getTime()));
            paramNumber++;
        }

        ResultSet results = s.executeQuery();

        List resultList = new ArrayList();
        while (results.next()) {
            LogEntry logEntry = new LogEntry();
            logEntry.setResourcePath(results.getString(DatabaseConstants.PATH_FIELD));
            logEntry.setUserName(results.getString(DatabaseConstants.USER_ID_FIELD));
            logEntry.setDate(new Date(results.getTimestamp(DatabaseConstants.RATED_TIME_FIELD).getTime()));
            logEntry.setAction(LogEntry.RATING);
            int rating = results.getInt(DatabaseConstants.RATING_FIELD);
            logEntry.setActionData(new Integer(rating).toString());

            resultList.add(logEntry);
        }

        return resultList;
    }
}
